@@ -7,6 +7,7 @@ from typing import Type, TypeVar, List, Generic, Iterable, Iterator, Callable, C
 
 import inject
 from is_empty import empty
+from selenium.common import NoSuchShadowRootException
 from selenium.webdriver import ActionChains
 from selenium.webdriver.common.by import By as SeleniumBy
 from selenium.webdriver.remote.webdriver import WebDriver
@@ -16,7 +17,7 @@ from selenium.webdriver.support.color import Color
 import paf.javascript as script
 from paf.assertion import StringAssertion, Format, BinaryAssertion, QuantityAssertion, RectAssertion, ASSERTION
 from paf.common import HasParent, Locator, Point, Rect, Property, Formatter, NotFoundException, NotUniqueException, \
-    WebdriverRetainer
+    WebdriverRetainer, RetryException, SubjectException
 from paf.control import retry
 from paf.dom import Attribute
 from paf.listener import Listener
@@ -139,6 +140,21 @@ class UiElement(
         else:
             return f"UiElement({self._by.__str__()})[{self._index}]"
 
+    @property
+    @abstractmethod
+    def _name(self) -> str:   # pragma: no cover
+        pass
+
+    @property
+    @abstractmethod
+    def _index(self) -> int:   # pragma: no cover
+        pass
+
+    @property
+    @abstractmethod
+    def _by(self) -> Locator:  # pragma: no cover
+        pass
+
     @abstractmethod
     def find(self, by: Locator, name: str = None) -> "UiElement":  # pragma: no cover
         pass
@@ -152,7 +168,7 @@ class UiElement(
         return UiElementAssertion(self, raise_exception=False)
 
     @abstractmethod
-    def _find_web_elements(self) -> ContextManager[List[WebElement]]: # pragma: no cover
+    def _find_web_elements(self) -> ContextManager[List[WebElement]]:  # pragma: no cover
         pass
 
     @contextmanager
@@ -179,13 +195,35 @@ class InteractiveUiElement(PageObject["InteractiveUiElement"], UiElementTests, U
     pass
 
 
-class InexistentUiElement(UiElement):
-
+class AbstractUiElement(UiElement, ABC):
     def __init__(self, name: str = None, parent: object = None):
-        self._name = name
-        self._parent = parent
-        self._by = By.id(None)
-        self._index = 0
+        self.__name = name
+        self.__parent = parent
+
+    @property
+    def _name(self) -> Locator:
+        return self.__name
+
+    @property
+    def _parent(self):
+        return self.__parent
+
+    @_parent.setter
+    def _parent(self, parent: HasParent):
+        self.__parent = parent
+
+
+class InexistentUiElement(AbstractUiElement):
+    def __init__(self, name: str = None, parent: object = None):
+        super().__init__(name, parent)
+
+    @property
+    def _by(self) -> Locator:
+        return By.id(None)
+
+    @property
+    def _index(self) -> int:
+        return 0
 
     @property
     def webdriver(self):
@@ -200,7 +238,7 @@ class InexistentUiElement(UiElement):
     def highlight(self, color: Color = Color.from_string("#0f0"), seconds: float = 2):
         pass
 
-    def take_screenshot(self) -> Path | None:
+    def take_screenshot(self, file_name: str = None) -> Path | None:
         pass
 
     def click(self):
@@ -247,8 +285,7 @@ class InexistentUiElement(UiElement):
         yield []
 
 
-class DefaultUiElement(UiElement):
-
+class DefaultUiElement(AbstractUiElement):
     def __init__(
             self,
             by: Locator,
@@ -266,10 +303,17 @@ class DefaultUiElement(UiElement):
 
         self._webdriver = webdriver
         self._ui_element = ui_element
-        self._by = by
-        self._index = index
-        self._parent = parent
-        self._name = name
+        self.__by = by
+        self.__index = index
+        super().__init__(name, parent)
+
+    @property
+    def _index(self) -> int:
+        return self.__index
+
+    @property
+    def _by(self) -> Locator:
+        return self.__by
 
     @property
     def webdriver(self):
@@ -305,7 +349,14 @@ class DefaultUiElement(UiElement):
                     self._webdriver.switch_to.frame(web_element)
                     web_elements = self._webdriver.find_elements(self._by.by, self._by.value)
                 else:
-                    web_elements = web_element.find_elements(self._by.by, self.__relative_selector(self._by))
+                    web_element_context = web_element
+                    try:
+                        web_element_context = web_element.shadow_root
+                    except NoSuchShadowRootException as e:
+                        pass
+
+                    web_elements = web_element_context.find_elements(self._by.by, self.__relative_selector(self._by))
+
                 yield self._filter_web_elements(web_elements)
 
         elif self._webdriver:
@@ -337,9 +388,10 @@ class DefaultUiElement(UiElement):
         try:
             retry(_sequence, lambda e: listener.action_failed(action_name, self, e))
             listener.action_passed(action_name, self)
-        except Exception as exception:
+        except SubjectException as exception:
+            exception.add_subject(self.name_path)
             listener.action_failed_finally(action_name, self, exception)
-            raise Exception(f"{self.name_path}: {exception}")
+            raise exception
 
     def click(self):
         self._web_element_action_sequence(lambda x: x.click(), "click")
@@ -479,7 +531,7 @@ class UiElementAssertion:
         return assertion_class(
             parent=self._ui_element,
             actual_supplier=_map,
-            name_supplier=lambda: f".{property_name} {Format.param(_map_failsafe())} ",
+            name_supplier=lambda: f".{property_name} {Format.param(_map_failsafe())}",
             raise_exception=self._raise,
         )
 
@@ -504,12 +556,18 @@ class UiElementAssertion:
         if isinstance(attribute, Attribute):
             attribute = attribute.value
 
-        return self._map_web_element_property(StringAssertion, lambda x: x.get_attribute(attribute),
-                                              f"attribute({attribute})")
+        return self._map_web_element_property(
+            StringAssertion,
+            lambda x: x.get_attribute(attribute),
+            f"attribute({attribute})"
+        )
 
     def css(self, property_name: str):
-        return self._map_web_element_property(StringAssertion, lambda x: x.value_of_css_property(property_name),
-                                              f"css({property_name}")
+        return self._map_web_element_property(
+            StringAssertion,
+            lambda x: x.value_of_css_property(property_name),
+            f"css({property_name}"
+        )
 
     def classes(self, *classes):
         return self.attribute(Attribute.CLASS).has_words(*classes)
@@ -544,7 +602,7 @@ class UiElementAssertion:
         return QuantityAssertion[int](
             parent=self._ui_element,
             actual_supplier=self._ui_element._count_elements,
-            name_supplier=lambda: f" count {Format.param(self._ui_element._count_elements())} ",
+            name_supplier=lambda: f" count {Format.param(self._ui_element._count_elements())}",
             raise_exception=self._raise,
         )
 
